<?php

namespace HCM\FPA\Controllers;

use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\FPA\Services\AprovacaoCalculoService;
use HCM\Geral\Services\ESocialService;
use HCM\Util\HCMParameters;
use HCM\Util\Util;
use HCM\Util\DateUtil;

class AprovacaoCalculoController extends Crud {
    
    protected $dataSourceNameget = 'fpa_assocevento_get';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AprovacaoCalculoService $aprovacaoCalculoService, ESocialService $eSocialService, EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->aprovacaoCalculoService = $aprovacaoCalculoService;
        $this->eSocialService = $eSocialService; 
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }
    
    public function getVinculoEventosNaoAprovados(DTO\Request\Filter $request, DTO\Response $response) {
        try{    
            set_time_limit(0); 
            $filters = Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
            $parameters = array();
            foreach ($filters as $position => $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                    }
                }
            }
            Util::getValueInFilterDataSourceByName($filters, array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRORG_PADRAO'), true);
            $conditions = Util::createQueryFiltered($filters);
            $result = $this->aprovacaoCalculoService->getVinculoEventosNaoAprovados($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getVinculoEventosNaoAprovados', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function aprovaCalculo(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows()[0];
            $retorno = [];
            $ferias = [];
            $rescisao = [];
            
            foreach($rows as $row){
                $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
                $row['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();
                $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVincMestreHistorico($row['NRORG'], $row['NRVINCULOM'], $row['DTMESCOMPETENC']);
                $row['NRESTRUTURAM'] = $vinculo->historico->getNrestrutlegal();
                
                if($row['NREVENTOM'] == '3030'){
                    $row['DTINIGOZOFERIAS'] = $row['DTOCORRENCIA'];
                    $row['DTFIMGOZOFERIAS'] = date('d/m/Y', strtotime(date('Y-m-d', strtotime(str_replace("/", "-", $row['DTOCORRENCIA'])))." +2 month"));
                    
                    $gozoFerias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->retornaGozosFeriasEntreDatas($row['NRORG'], $row['NRVINCULOM'], DateUtil::getDatadeString($row['DTINIGOZOFERIAS']), DateUtil::getDatadeString($row['DTFIMGOZOFERIAS']));
                    
                    if (!empty($gozoFerias)) {
                        $row['NRGOZOFERIAS'] = $gozoFerias[0]? $gozoFerias[0]->getNrgozoferias() : null;
                        $row['NMEVENTO'] = 'S-2230';
                        $row['NRCHAVE'] = $row['NRGOZOFERIAS'];
                        $row['NRTABELA'] = 2;
                    }
                    
                    array_push($ferias, $row);
                }
                
                if($row['NREVENTOM'] == '3050'){
                
                    if($vinculo->getNrtipovinculom() == 1){
                        $row['NMEVENTO'] = 'S-2299';
                        $row['NRCHAVE'] = $vinculo->getNrvinculom();
                    }else{
                        $row['NMEVENTO'] = 'S-2399';
                        $row['NRCHAVE'] = $vinculo->getNrvinculom();
                    }
                    
                    array_push($rescisao, $row);
                }
                
                $dados = $this->aprovacaoCalculoService->aprovaCalculo($row);
                array_push($retorno, $dados);
            }
            
            if(array_key_exists('IDENVIAESOCIAL', $rows[0]) && $rows[0]['IDENVIAESOCIAL'] == 'S'){
                if(count($ferias) > 0){
                    $retornoEsocial2230 = [];
                    foreach($ferias as $itemFerias){
                        $this->eSocialService->envioEsocialDescentralizado($itemFerias, $response);
                        
                        $dataSets = $response->getDataSets();
                        
                        foreach($dataSets as $dataSet){
                            if($dataSet->getDataSourceName() == 'retornoEsocial'){
                                $retornoEsocial2230[0] = ['VINCULO'  => $itemFerias['NRVINCULOM'],
                                                          'RETORNO' =>$dataSet->getRows()[0]];
                            }
                        }
                    }
                    $response->addDataSet((new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial2230', $retornoEsocial2230)));
                }
                
                if(count($rescisao) > 0){
                    $retornoEsocial2299 = [];
                    foreach($rescisao as $itemRescisao){
                        $this->eSocialService->envioEsocialDescentralizado($itemRescisao, $response);
                        
                        $dataSets = $response->getDataSets();
                        
                        foreach($dataSets as $dataSet){
                            if($dataSet->getDataSourceName() == 'retornoEsocial'){
                                $retornoEsocial2299[0] = ['VINCULO'  => $itemRescisao['NRVINCULOM'],
                                                          'RETORNO' =>$dataSet->getRows()[0]];
                            }
                        }
                    }
                    
                    $response->addDataSet((new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial2299', $retornoEsocial2299)));
                }
                
            }
            
            if($retorno){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('aprovaCalculo', $retorno));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}