<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\Util\Messages;

class GrupoIncidenciaController extends Crud {

    protected $grupoIncidenciaService;
    protected $dataSourceName = 'fpa_grpincide';
    protected $dataSourceName1 = 'fpa_grpitem_get';
    protected $entityManager;
    protected $hcmParameters;
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\FPA\Services\GrupoIncidenciaService $grupoIncidenciaService, EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->grupoIncidenciaService = $grupoIncidenciaService;
        parent::__construct($dataSourceManager);
    }

    public function getGrupoIncidencia(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getItemGrupoIncidencia(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setGrupoIncidencia(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRGRPINCIDE', $row)) {
                    if ($row['NRGRPINCIDE'] === '') {
                        $row['NRGRPINCIDE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_GRPINCIDE', 1, 12, true); 
                    }
                } else {
                    $row['NRGRPINCIDE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_GRPINCIDE', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function replicaGrupoIncidencia(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->grupoIncidenciaService->replicaGrupo($row);
            }
            $response->addNotification(new DTO\Response\Notification("Grupo de Incidência replicado com sucesso!",  DTO\Response\Notification::TYPE_SUCCESS));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setItemGrupoIncidencia(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "fpa_grpitem";

            foreach ($request->getDataSet()->getRows() as &$row) {
                if ($row['__is_new'] == true && self::existeItemGrupoDeIncidencia($row['NRITEM'], $row['NRGRPINCIDE'])) {
                    $msg = 'Existe item com esse mesmo Nr. Item';
                    $response->addNotification(new Notification($msg, Notification::TYPE_ERROR));
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($msg, DTO\Response\Message::TYPE_ERROR));
                    return;
                }

                if ($row['__is_new'] == true) {
                    $row['NRGRPITEM'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_GRPITEM', 1, 12, true);
                }

                if (!array_key_exists('NREVENTOM', $row)) {
                    $row['NREVENTOM'] = null;
                }
            }

            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function existeItemGrupoDeIncidencia($nrItem, $nrGrpIncide): bool
    {
        $params = array(
            'nrorg' => array(
                $this->hcmParameters->getNrorgPadrao(),
                $this->hcmParameters->getNrorgTrabalho()
            ),
            'nrgrpincide' => $nrGrpIncide,
            'nritem' => $nrItem
        );

        $itemGrupoDeIncidencia = $this->entityManager->getRepository(Repositories::FPA_GRPITEM)->findOneBy($params);

        return !empty($itemGrupoDeIncidencia);
    } 
    
    public function deleteGrupoIncidencia(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->grupoIncidenciaService->deleteGrupoIncidencia($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }
    
    public function deleteItemGrupoIncidencia(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "fpa_grpitem";
        parent::delete($request, $response);
    }

    public function setGrupoIncidenciaPopUpParametroFolha(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach($request->getDataSet()->getRows() as &$row){
                $dtAtual = DateUtil::getDataAtual();
                $cdOperador = $this->hcmParameters->getCdoperador();
                $nrOrg = $this->hcmParameters->getNrorgTrabalho();
                
                $row['DTFIMVIGENCIA'] = null;
                $row['DTINIVIGENCIA'] = date_create_from_format('d/m/Y', $row['DTINIVIGENCIA']);
                $row['DTINCLUSAO'] = $dtAtual;
                $row['CDOPERINCLUSAO'] = $cdOperador;
                $row['NRORGINCLUSAO'] = $nrOrg;
                $row['DTULTATU'] = $dtAtual;
                $row['CDOPERULTATU'] = $cdOperador;
                $row['NRORGULTATU'] = $nrOrg;
                $row['NREVENTOM'] = !empty($row['NREVENTOM']) ? $row['NREVENTOM'] : null;
                
                $nrgrpincide = $this->grupoIncidenciaService->setGrupoIncidenciaPorArray($row);
                
                $this->grupoIncidenciaService->setItensGrupoIncidenciaPopUpParametroFolha($nrgrpincide, $row['INSSTERCEIROS'], $row['INSSEMPRESA'], $row['INSSSAT'], $row['INSSPARTEEMPRESAPROLABORE'], $row['INSSPARTEEMPRESAAUTONOMO'], $row['NREVENTOM'], $row['LIQUIDOGUIAGPS']);
                
                $response->addNotification(new DTO\Response\Notification("Grupo Incidência cadastrado com sucesso!",  DTO\Response\Notification::TYPE_SUCCESS));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('NRGRPINCIDE', array($nrgrpincide)));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 