<?php

namespace HCM\PTO\Logic\AFD\Portaria671;

/**
 * Portaria 671
 * Layout do AFD. Responsavel por gerenciar registros especificos da portaria e gerar seus conteudos
 * Layout definido em http://www.normaslegais.com.br/legislacao/portaria-mtp-671-2021.htm
 */

use HCM\PTO\Logic\AFD\Layout as LayoutAFD;
use HCM\PTO\Logic\AFD\Rodape as RodapeAFD;
use HCM\PTO\Logic\AFD\Cabecalho as CabecalhoAFD;
use HCM\PTO\Logic\AFD\Parametros;
use HCM\Util\Util;

class Layout extends LayoutAFD {
    /** @var Registro[] */
    private $registros;
    /** @var CabecalhoAFD */
    private $cabecalho;
    /** @var RodapeAFD */
    private $rodape;
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Parametros $parametros) {
        parent::__construct($entityManager, $parametros);
        $this->registros = $this->criaRegistros();
        $this->cabecalho = $this->criaCabecalho();
        $this->rodape = $this->criaRodape();
    }
    
    /**
     * Portaria 671
     * Gera o conteudo do layout a partir de registros especificos
     * @return string
     */
    public function gerar(): string {
        $conteudo = $this->cabecalho->geraConteudo() . Util::CRLF .
            $this->geraRegistros() . Util::CRLF . $this->rodape->geraConteudo();
        return Util::removeTextBlankLines($conteudo);
    }
    
    /**
     * Portaria 671
     * Obtem o conteudo dos registros do layout concatenados separados por quebra de linha
     * @return string
     */
    public function geraRegistros(): string {
        $registros = $this->obtemConteudoRegistros();
        return array_reduce($registros, function ($acc, $item) {
            return $acc . Util::CRLF . $item;
        });
    }
    
    /**
     * Portaria 671
     * Obtem o conteudo dos registros do layout
     * @return string[]
     */
    public function obtemConteudoRegistros(): array {
        $conteudoRegistros = [];
        foreach ($this->registros as $registro) {
            $conteudoRegistros[] = $registro->geraConteudo();
        }
        return $conteudoRegistros;
    }
    
    /**
     * Portaria 671
     * Instancia os registros que estarao presentes no arquivo
     * @return Registro[]
     */
    public function criaRegistros(): array {
        return [
            new RegistroMarcacao($this->getEntityManager(), $this->getParametros()),
        ];
    }
    
    /**
     * Portaria 671
     * Instancia o cabecalho que estara presente no arquivo
     * @return CabecalhoAFD
     */
    public function criaCabecalho(): CabecalhoAFD {
        return new Cabecalho($this->getEntityManager(), $this->getParametros());
    }
    
    /**
     * Portaria 671
     * Instancia o rodape que estara presente no arquivo
     * @return RodapeAFD
     */
    public function criaRodape(): RodapeAFD {
        return new Rodape(new ContadorRegistros($this->registros));
    }
}